/*
 * FW_UV_driver.c
 *
 * Created: 12-Jul-16 14:30:46
 * Author : maticpi
 *
 * Tipke in UART1 poiljajo ukaze na LED driver. 
 * LCD se ne odziva na tipke, temve? le na status sporo?ila, ki jih poilja LED driver - tako vedno ve kaj dela LED, ne pa kaj si ravnokar naredil.
 */ 

#include "main.h"

uint8_t UVtype=2;	//makes it refresh when switched ON;
uint8_t reportedUVtype;
uint8_t UVstatus;
uint8_t mode;
uint32_t statusTimeStamp;
uint32_t ElapsedTime;
float LEDpowerDensityHigh;		//Units are mW/m2 (CIE weighted)
float LEDpowerDensityMedium;
float LEDpowerDensityLow;
int TempLED, TempAlu;
float doseCountdown;
int32_t timerCountdown;
float K;

int main(void)
{
	Init_IO();
	systime_init();
	UART0_Init();	//LED
	UART1_Init();	//Bluetooth
	LCD_Init();
	stdout = &LCD_str;
	sei();
	LED_Power_ON();
    /* Replace with your application code */
    while (1) 
    {
		ProcessMessagesFromLEDdriver();
		ProcessMessagesFromBluetooth();
		ProcessKeyboardEvents();
		UpdateStatus();
    }
}

void ProcessMessagesFromLEDdriver()
{
	char data; 
	static char msg[16];
	static uint8_t receivingLongMessage=LONG_MSG_NOT;
	static int i;
	
	if (UART0_DataReady())
	{
		data=UART0_GetByte();
		switch (receivingLongMessage)
		{
		case LONG_MSG_STATUS:
			msg[i]=data;
			i++;
			if (i == 12)	//end of message
			{
				reportedUVtype = msg[0];
				UVstatus = msg[1] & 0x0f;
				mode = (msg[1]>>4) & 0x0f;
				ElapsedTime  = (uint32_t)msg[2]<<24;
				ElapsedTime += (uint32_t)msg[3]<<16;
				ElapsedTime += (uint32_t)msg[4]<<8;
				ElapsedTime += (uint32_t)msg[5];
				if (msg[6] == 128) TempLED = -1;
				else TempLED = msg[6];
				if (msg[7] == 128) TempAlu = -1;
				else TempAlu = msg[7];
				if (mode == MODE_TIMER)
				{
					timerCountdown  = (uint32_t)msg[8]<<24;
					timerCountdown |= (uint32_t)msg[9]<<16;
					timerCountdown |= (uint32_t)msg[10]<<8;
					timerCountdown |= (uint32_t)msg[11];
				}
				else if (mode == MODE_DOSE)
				{
					*(uint32_t *)&doseCountdown  = (uint32_t)msg[8]<<24;
					*(uint32_t *)&doseCountdown += (uint32_t)msg[9]<<16;
					*(uint32_t *)&doseCountdown += (uint32_t)msg[10]<<8;
					*(uint32_t *)&doseCountdown += (uint32_t)msg[11];
				}

				receivingLongMessage=LONG_MSG_NOT;
				statusTimeStamp = GetSysTick();
			}
			break;
		case LONG_MSG_CONFIG:
			msg[i]=data;
			i++;
			if (i == 16)	//end of message
			{
				*(uint32_t *)&LEDpowerDensityLow  = (uint32_t)msg[0]<<24;
				*(uint32_t *)&LEDpowerDensityLow += (uint32_t)msg[1]<<16;
				*(uint32_t *)&LEDpowerDensityLow += (uint32_t)msg[2]<<8;
				*(uint32_t *)&LEDpowerDensityLow += (uint32_t)msg[3];
				*(uint32_t *)&LEDpowerDensityMedium  = (uint32_t)msg[4]<<24;
				*(uint32_t *)&LEDpowerDensityMedium += (uint32_t)msg[5]<<16;
				*(uint32_t *)&LEDpowerDensityMedium += (uint32_t)msg[6]<<8;
				*(uint32_t *)&LEDpowerDensityMedium += (uint32_t)msg[7];
				*(uint32_t *)&LEDpowerDensityHigh  = (uint32_t)msg[8]<<24;
				*(uint32_t *)&LEDpowerDensityHigh += (uint32_t)msg[9]<<16;
				*(uint32_t *)&LEDpowerDensityHigh += (uint32_t)msg[10]<<8;
				*(uint32_t *)&LEDpowerDensityHigh += (uint32_t)msg[11];
				*(uint32_t *)&K  = (uint32_t)msg[12]<<24;
				*(uint32_t *)&K += (uint32_t)msg[13]<<16;
				*(uint32_t *)&K += (uint32_t)msg[14]<<8;
				*(uint32_t *)&K += (uint32_t)msg[15];

				receivingLongMessage=LONG_MSG_NOT;
			}
			break;
		default:
			switch (data)
			{
				case 0xFF: receivingLongMessage = LONG_MSG_STATUS; i=0; break;
				case 0xFE: receivingLongMessage = LONG_MSG_CONFIG; i=0; break;
				default: UART1_put(data);
				break;
			}
			break;
		}
	}
}

void ProcessMessagesFromBluetooth()
{
	char cmd;
	if (UART1_DataReady())
	{
		cmd=UART1_GetByte();
		UART0_put(cmd);
	}
}

void UpdateStatus()
{
	uint8_t type;
	static uint32_t timeBlink;
	static uint8_t oldStatus=LEDS_STATUS_REFRESH;
	static uint32_t oldElapsedTime=0xFFFFFFFF;	
	static int32_t oldTimerCountdown;
	int h,m,s;
	static float dose,oldDoseCountdown;
	static int oldTempLED;

	if (IsUVA()) type = TYPE_UVA;
	else type = TYPE_UVB;
	if (GetSysTick() > (statusTimeStamp + 1000)) UVstatus = LEDS_STATUS_OUTDATED;
	
	//Status LEDs
	switch (UVstatus)
	{
		case LEDS_OFF:				PORTB &= 0xc0; break; 
		case LEDS_ON10:				PORTB &= 0xc0; PORTB |= 0x03; break;
		case LEDS_ON30:				PORTB &= 0xc0; PORTB |= 0x0F; break;
		case LEDS_ON100:			PORTB &= 0xc0; PORTB |= 0x3F; break;
		case LEDS_ON10_OVERTEMP:
			PORTB &= 0xc3;
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x03;
			break;
		case LEDS_ON30_OVERTEMP:	
			PORTB &= 0xcF;
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x0F;
			break;
		case LEDS_ON100_OVERTEMP:
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x3F;
			break;
		case LEDS_STATUS_OUTDATED:
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x20;
			break;
		case LEDS_STATUS_UNKNOWN:
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x30;
			break;
		default:
			if (Has_X_MillisecondsPassed(300,&timeBlink)) PORTB ^= 0x3F;
			break;
	}
	//LCD LINE 0: type, power
	if (type != UVtype)
	{
		UVtype = type;
		LCD_GotoXY(0,0);
		if (UVtype == TYPE_UVA) printf("UV-A");
		else printf("UV-B");
	}
	if (oldStatus != UVstatus)
	{
		oldStatus = UVstatus;
		LCD_GotoXY(5,0);
		switch (UVstatus)
		{
			case LEDS_ON100:			printf(" HIGH "); break;	//replace with W/m2(CIE)
			case LEDS_ON30:				printf("MEDIUM"); break;	//replace with W/m2(CIE)
			case LEDS_ON10:				printf("  LOW "); break;	//replace with W/m2(CIE)
			case LEDS_ON100_OVERTEMP:	printf("HOT %dC %dC",TempLED,TempAlu); break;
			case LEDS_ON30_OVERTEMP:	printf("HOT %dC %dC",TempLED,TempAlu); break;
			case LEDS_ON10_OVERTEMP:	printf("HOT %dC %dC",TempLED,TempAlu); break;
			case LEDS_OFF:				printf("  OFF "); break;
			case LEDS_NO_POWER:			printf("SWITCH"); break;
			case LEDS_ERROR:			printf(" ERROR"); break;
			case LEDS_STATUS_OUTDATED:	printf("NO LED"); break;
			default: printf("Err "); break;
		}
	}
	if ((UVstatus == LEDS_ON10) ||
		(UVstatus == LEDS_ON30) ||
		(UVstatus == LEDS_ON100) ||
		(UVstatus == LEDS_OFF))
	{
		if (mode == MODE_TIMER)		
		{
			if (timerCountdown != oldTimerCountdown)
			{
				LCD_GotoXY(11,0);
				if (timerCountdown <= 99999) printf("%5ld",timerCountdown);
				else printf("*****");
				oldTimerCountdown = timerCountdown;
			}
		}
		else if (mode == MODE_DOSE)
		{
			if (doseCountdown != oldDoseCountdown)
			{
				LCD_GotoXY(11,0);
				if (doseCountdown > 99999.0) printf("*****");
				else if (doseCountdown >= 1000.0) printf("%5.0f",doseCountdown);
				else if (doseCountdown >= 100.0) printf("%5.1f",doseCountdown);
				else printf("%5.2f",doseCountdown);
				oldDoseCountdown = doseCountdown;
			}
		}
		else 
		{
			if (oldTempLED != TempLED)
			{
				LCD_GotoXY(11,0);
				printf("%4dC",TempLED);
				oldTempLED=TempLED;
			}
		}
	}
	//LCD LINE 1: time, calculated dose
	if (oldElapsedTime != ElapsedTime)
	{
		oldElapsedTime = ElapsedTime;
		h=ElapsedTime/3600UL;
		m=(ElapsedTime - h*3600UL)/60UL;
		s=(int)(ElapsedTime - h*3600UL - m*60UL);
		switch (UVstatus)
		{
			case LEDS_ON10: dose = ElapsedTime * LEDpowerDensityLow * K / 1000.0; break;
			case LEDS_ON30: dose = ElapsedTime * LEDpowerDensityMedium * K / 1000.0; break;
			case LEDS_ON100: dose = ElapsedTime * LEDpowerDensityHigh * K / 1000.0; break;
			default:	//if off or error, do not recalculate the dose,
				break;	// so that the last value remains on the screen
		}
		LCD_GotoXY(0,1);
		printf("%02d:%02d:%02d %6.2f",h,m,s,dose);	//dose units are J/m2 (CIE weighted)
	}
}

void ProcessKeyboardEvents()
{
	char kbd;
	static uint32_t timeKBD;
	if (Has_X_MillisecondsPassed(10,&timeKBD))
	{
		ReadKBD();
		kbd = GetKey();
		switch (kbd)
		{
			case 4: UART0_put('3'); break; //On - 100%
			case 3: UART0_put('2'); break; //On - 30%
			case 2: UART0_put('1'); break; //On - 10%
			case 1: UART0_put('0'); break; //Off
			default: break;
		}
	}
}

void Init_IO()
{
	PORTA=0xFF;
	PORTB=0xC0;
	PORTC=0xFF;
	PORTD=0xBF;
	LED_Power_OFF();	//Set port value before port direction is changed
	//1-out, 0-in
	DDRA=0x00;	// LLLL KK?v	L=LCD, K=Key, v=UV LED voltage - analog input
	DDRB=0x3F;	// KKss ssss	K=Key, s=status LED 
	DDRC=0x00;	// LL?? ???L	L=LCD
	DDRD=0x90;	// BI?F UART	B=backlight,I=UVAB_ID, F=PowerOnOff, UARTS (initialized elsewhere)
}