/*
 * TLC59401.c
 *
 * Created: 12-Jul-16 15:07:31
 *  Author: maticpi
 */ 

#include "TLC59401.h"

uint16_t GSdata[16*TLC59401_COUNT];
uint8_t DCdata10[16*TLC59401_COUNT];
uint8_t DCdata30[16*TLC59401_COUNT];
uint8_t DCdata100[16*TLC59401_COUNT];
float CalData[3];
uint8_t status[24*TLC59401_COUNT];
uint8_t LEDStatus=LEDS_OFF;
uint8_t CycleTypeStatus=LAST_CYCLE_DC;
uint8_t PWM;

void TLC59401_Init()
{
	int i;

	//GSCLK
	GSClkLow();
	TLC59401_GSCLK_DDR |= (1<<TLC59401_GSCLK_BIT);
	
	//BLANK
	OutputsOff();
	TLC59401_BLANK_DDR |= (1<<TLC59401_BLANK_BIT);

	//MODE
	ModeDC();
	TLC59401_MODE_DDR |= (1<<TLC59401_MODE_BIT);
	
	//ERR
	TLC59401_ERR1_DDR &= ~(1<<TLC59401_ERR1_BIT);	//input
	TLC59401_ERR1_PORT |= (1<<TLC59401_ERR1_BIT);	//enable pull-up
	TLC59401_ERR2_DDR &= ~(1<<TLC59401_ERR2_BIT);	//input
	TLC59401_ERR2_PORT |= (1<<TLC59401_ERR2_BIT);	//enable pull-up
	
	//XLAT
	LatchOff();
	TLC59401_XLAT_DDR |= (1<<TLC59401_XLAT_BIT);

	//MOSI - User defined output
	PORTB &= ~(1<<5);
	DDRB  |=  (1<<5);
	//MISO - Automatically set as input
	PORTB &= ~(1<<6);
	DDRB  &= ~(1<<6);
	//SCK - User defined output
	PORTB &= ~(1<<7);
	DDRB  |=  (1<<7);
	//SS - Set as output to prevent it from interfering with the operation of SPI
	DDRB  |=  (1<<4);
	//SPI
	SPSR0 = (0<<SPI2X0);	//NOT Double speed
	SPCR0 = (1<<SPE0) | (1<<MSTR0) | (0<<CPHA0) | (3<<SPR00);	//Enable, Master, minMax frequency
	//SPDR0 - SPI data register
	
	
	//Set GS, then set DC
	for (i=0; i<16*TLC59401_COUNT; i++) GSdata[i]=0;		//Set All to zero
	GSdata[6] = 0xFFF;
	GSdata[4] = 0xFFF;
	GSdata[2] = 0xFFF;
	GSdata[31] = 0xFFF;
	GSdata[29] = 0xFFF;
	GSdata[27] = 0xFFF;
	GSdata[7] = 0xFFF;
	GSdata[0] = 0xFFF;
	GSdata[15] = 0xFFF;
	GSdata[16] = 0xFFF;
	GSdata[18] = 0xFFF;
	GSdata[25] = 0xFFF;
	GSdata[9] = 0xFFF;
	GSdata[11] = 0xFFF;
	GSdata[13] = 0xFFF;
	GSdata[20] = 0xFFF;
	GSdata[22] = 0xFFF;
	GSdata[23] = 0xFFF;
 	
	GetDataFromEEPROM();
	SetGSandGetStatus(GSdata,status);
	SetDC(DCdata10);
}

void EnableLockIn()
{
	OutputsOff();
	GSdata[24]=0x07FF;
	GSdata[6] = 0x7FF;
	GSdata[4] = 0x7FF;
	GSdata[2] = 0x7FF;
	GSdata[31] = 0x7FF;
	GSdata[29] = 0x7FF;
	GSdata[27] = 0x7FF;
	GSdata[7] = 0x7FF;
	GSdata[0] = 0x7FF;
	GSdata[15] = 0x7FF;
	GSdata[16] = 0x7FF;
	GSdata[18] = 0x7FF;
	GSdata[25] = 0x7FF;
	GSdata[9] = 0x7FF;
	GSdata[11] = 0x7FF;
	GSdata[13] = 0x7FF;
	GSdata[20] = 0x7FF;
	GSdata[22] = 0x7FF;
	GSdata[23] = 0x7FF;
	if (LEDStatus == LEDS_ON10) {DCdata10[24]=0x3F;	SetDC(DCdata10);}
	else if (LEDStatus == LEDS_ON30) {DCdata30[24]=0x3F;	SetDC(DCdata30);}
	else if (LEDStatus == LEDS_ON100) {DCdata100[24]=0x3F;	SetDC(DCdata100);}
	else {printf("LEDS are off, lock-in mode aborted.\n"); return;}
	SetGSandGetStatus(GSdata,status);
	EnablePWMmode();
	PWM=1;	
}

void DisableLockIn()
{
	int i;
	printf("disable PWM for lock-in.\n");
	DisablePWMmode();
	OutputsOff();
	for (i=0; i<16*TLC59401_COUNT; i++) GSdata[i]=0;		//Set All to zero
	GSdata[6] = 0xFFF;
	GSdata[4] = 0xFFF;
	GSdata[2] = 0xFFF;
	GSdata[31] = 0xFFF;
	GSdata[29] = 0xFFF;
	GSdata[27] = 0xFFF;
	GSdata[7] = 0xFFF;
	GSdata[0] = 0xFFF;
	GSdata[15] = 0xFFF;
	GSdata[16] = 0xFFF;
	GSdata[18] = 0xFFF;
	GSdata[25] = 0xFFF;
	GSdata[9] = 0xFFF;
	GSdata[11] = 0xFFF;
	GSdata[13] = 0xFFF;
	GSdata[20] = 0xFFF;
	GSdata[22] = 0xFFF;
	GSdata[23] = 0xFFF;
	DCdata10[24]=0;
	DCdata30[24]=0;
	DCdata100[24]=0;
	SetDC(DCdata10);
	SetGSandGetStatus(GSdata,status);
	PWM=0;
}

void EnablePWMmode()
{
	//Order is important!!! BLANK must lag at least 10ns behind GSCLK
	//Setup timer 0 - GSCLK
	TCCR2A = (0<<COM0A0) |(2<<COM0B0) | (3<<WGM00);	//WGM0x=7
	TCCR2B = (1<<WGM02)	| (3<<CS00);	// 1:1 
	OCR2A = 0x03;
	OCR2B = 0x01;
	//Setup timer 1 - BLANK
	TCCR1A = (3<<COM1A0) |(0<<COM1B0) | (2<<WGM10);	//WGM1x=14
	TCCR1B = (3<<WGM12)	| (3<<CS10);	// 1:1 
	ICR1 = 0x1FFF;	//2*4096-1
	OCR1A = 0x1FFE;
}

void DisablePWMmode()
{
	#warning "Not finished"
	//Disable timer 1 - BLANK
	TCCR1A = (0<<COM1A0) |(0<<COM1B0) | (2<<WGM10);	//Normal port operation
	TCCR1B = (3<<WGM12)	| (0<<CS10);	// Clk src off
	TCCR2A = (0<<COM0A0) |(0<<COM0B0) | (3<<WGM00);	//Normal port operation
	TCCR2B = (1<<WGM02)	| (0<<CS00);	// Clk src off
	GSClkLow();
	OutputsOff();
}

void PulseSck()
{
	PORTB &= ~(1<<7);		//SCK=0;
	SPCR0 &= ~(1<<SPE0);	//Disable SPI
	PORTB |= (1<<7);		//SCK=1;
	TLC59401_SPI_SCK_DELAY();
	PORTB &= ~(1<<7);		//SCK=0;
	SPCR0 |= (1<<SPE0);		//Enable SPI
}

void SetGSandGetStatus(uint16_t* data, uint8_t* stat)
{
	//N*16 channels, 12 bits per channel, MSB first
	uint8_t tmpStatus[24*TLC59401_COUNT];
	int i,j;
	
	LatchOff();	//not really necessary
	ModeGS();
	if (CycleTypeStatus == LAST_CYCLE_GS) PulseSck();
	
	for (j=0, i=16*TLC59401_COUNT-1; i>=0; i-=2)
	{
		SPDR0 = (data[i] >> 4) & 0xff;
		while(!(SPSR0 & (1<<SPIF0)));
		tmpStatus[j++]=SPDR0;
		SPDR0 = ((data[i] << 4) & 0xff) | ((data[i-1] >> 8) & 0xff);
		while(!(SPSR0 & (1<<SPIF0)));
		tmpStatus[j++]=SPDR0;
		SPDR0 = data[i-1] & 0xff;
		while(!(SPSR0 & (1<<SPIF0)));
		tmpStatus[j++]=SPDR0;
	}
	LatchOn();
	TLC59401_LATCH_DELAY();
	LatchOff();
	if (CycleTypeStatus == LAST_CYCLE_GS)
	{
		for (j=0; j<24*TLC59401_COUNT;j++) stat[j]=tmpStatus[j];
	}
	CycleTypeStatus=LAST_CYCLE_GS;
}

void SetDC(uint8_t* data)
{
	//16 channels, 6 bits per channel, MSB first
	int i;
	LatchOff();	//not really necessary
	ModeDC();
	
	#warning "Dummy byte solution. Is there another way?"
	//Send one dummy byte first
	SPDR0 = 0;
	while(!(SPSR0 & (1<<SPIF0)));
	
	for (i=16*TLC59401_COUNT-1; i>=0; i-=4)
	{
		SPDR0 = ((data[i] << 2) & 0xfc) | ((data[i-1] >> 4) & 0x03);
		while(!(SPSR0 & (1<<SPIF0)));
		SPDR0 = ((data[i-1] << 4) & 0xf0) | ((data[i-2] >> 2) & 0x0f);
		while(!(SPSR0 & (1<<SPIF0)));
		SPDR0 = ((data[i-2] << 6) & 0xc0) | (data[i-3] & 0x3f);
		while(!(SPSR0 & (1<<SPIF0)));
	}
	LatchOn();
	TLC59401_LATCH_DELAY();
	LatchOff();
	CycleTypeStatus=LAST_CYCLE_DC;
}

void GetDataFromEEPROM()
{
	uint8_t data;
	int i;
	
	data = eeprom_read_byte ((uint8_t *)EEPROM_ADDRESS_DC_DATA);
	if (data > 0x3F)	//if EEPROM is empty
	{
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata10[i]=0;		//Set All to zero
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata30[i]=0;		//Set All to zero
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata100[i]=0;		//Set All to zero
		for (i=0; i<3; i++) CalData[i]=0;
	}
	else
	{	//Get data from EEPROM
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata10[i]=eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+i));
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata30[i]=eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+16*TLC59401_COUNT+i));
		for (i=0; i<16*TLC59401_COUNT; i++) DCdata100[i]=eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+32*TLC59401_COUNT+i));
		for (i=0; i<3; i++) CalData[i]=eeprom_read_float ((float *)(EEPROM_ADDRESS_CAL_DATA + 4*i));
	}
}

void SaveDataToEEPROM()
{
	int i;
	
	for (i=0; i<16*TLC59401_COUNT; i++) 
		eeprom_write_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+i), DCdata10[i]);
	for (i=0; i<16*TLC59401_COUNT; i++)
		eeprom_write_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+16*TLC59401_COUNT+i), DCdata30[i]);
	for (i=0; i<16*TLC59401_COUNT; i++)
		eeprom_write_byte ((uint8_t *)(EEPROM_ADDRESS_DC_DATA+32*TLC59401_COUNT+i), DCdata100[i]);
	for (i=0; i<3; i++)
		eeprom_write_float ((float *)(EEPROM_ADDRESS_CAL_DATA+4*i), CalData[i]);
}